@@ -2,7 +2,7 @@
 PC-BASIC - devicebase.py
 Devices, Files and I/O operations
 
-(c) 2013--2022 Rob Hagemans
+(c) 2013--2023 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -196,6 +196,9 @@ class RawFile(object):
         with safe_io():
             self._fhandle.write(s)
 
+    def flush(self):
+        """Stub for compatibility with file-like objects."""
+
 
 #################################################################################
 # Text file base
