@@ -2,7 +2,7 @@
 PC-BASIC - program.py
 Program buffer utilities
 
-(c) 2013--2022 Rob Hagemans
+(c) 2013--2023 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -45,26 +45,27 @@ class Program(object):
         offset_val, p = 0, 0
         output = []
         for key in sorted(self.line_numbers.keys())[1:]:
+            next_p = self.line_numbers[key]
             offset, linum = code[p+1:p+3], code[p+3:p+5]
             last_offset = offset_val
             offset_val = struct.unpack('<H', offset)[0] - (self.code_start + 1)
             linum_val, = struct.unpack('<H', linum)
-            output.append('%s %s (+%03d) %s [%05d] %s' % (
+            output.append(b'%s %s (+%03d) %s [%05d] %s' % (
                 binascii.hexlify(code[p:p+1]),
                 binascii.hexlify(offset),
                 offset_val - last_offset,
                 binascii.hexlify(code[p+3:p+5]),
                 linum_val,
-                binascii.hexlify(code[p+5:])
+                binascii.hexlify(code[p+5:next_p])
             ))
-            p = self.line_numbers[key]
-        output.append('%s %s (ENDS) %s %s' % (
+            p = next_p
+        output.append(b'%s %s (ENDS) %s %s' % (
             binascii.hexlify(code[p:p+1]),
             binascii.hexlify(code[p+1:p+3]),
             binascii.hexlify(code[p+3:p+5]),
             binascii.hexlify(code[p+5:])
         ))
-        return '\n'.join(output)
+        return b'\n'.join(output).decode('ascii', 'replace')
 
     def size(self):
         """Size of code space """
@@ -237,34 +238,25 @@ class Program(object):
         # update line number dict
         self.update_line_dict(startpos, afterpos, 0, deleteable, beyond)
 
-    def edit(self, console, from_line, bytepos=None):
+    def edit(self, console, from_line, target_bytepos):
         """Output program line to console and position cursor."""
         if self.protected:
             console.write(b'%d\r' % (from_line,))
             raise error.BASICError(error.IFC)
         # list line
         self.bytecode.seek(self.line_numbers[from_line]+1)
-        _, output, textpos = self.lister.detokenise_line(self.bytecode, bytepos)
-        # no newline to avoid scrolling on line 24
-        console.list_line(bytes(output), newline=False)
-        # find row, column position for textpos
-        newlines, c = 0, 0
-        pos_row, pos_col = 0, 0
-        if not output:
-            return
-        for i, byte in enumerate(output):
-            c += 1
-            if byte == ord(b'\n') or c > console.width:
-                newlines += 1
-                c = 0
-            if i == textpos:
-                pos_row, pos_col = newlines, c
-        if textpos > i:
-            pos_row, pos_col = newlines, c + 1
-        if bytepos:
-            console.set_pos(console.current_row-newlines+pos_row, pos_col)
+        _, output, byte_to_text_positions = self.lister.detokenise_line(self.bytecode)
+        # find text position from byte position
+        if target_bytepos is None:
+            textpos = None
         else:
-            console.set_pos(console.current_row-newlines, 1)
+            textpos = min(
+                _textpos
+                for _bytepos, _textpos in byte_to_text_positions
+                if target_bytepos <= _bytepos
+            )
+        # no newline to avoid scrolling on line 24
+        console.list_line(bytes(output), newline=False, set_text_position=textpos)
 
     def renum(self, console, new_line, start_line, step):
         """Renumber stored program."""
@@ -427,7 +419,10 @@ class Program(object):
         code = self.bytecode.getvalue()
         try:
             return ord(code[offset:offset+1])
-        except IndexError:
+        except IndexError: # pragma: no cover
+            # variable memory starts immediately after the stored program
+            # ao memory access beyond the size of the program should not arrive here
+            logging.debug('PEEK beyond program size handled by Program class.')
             return -1
 
     def get_memory_block(self, offset, length):
@@ -440,17 +435,20 @@ class Program(object):
         """Change program code."""
         if not self.allow_code_poke:
             logging.warning('Ignored POKE into program code')
+            return
+        offset -= self.code_start
+        loc = self.bytecode.tell()
+        # move pointer to end
+        self.bytecode.seek(0, 2)
+        if offset > self.bytecode.tell(): # pragma: no cover
+            # variable memory starts immediately after the stored program
+            # ao memory access beyond the size of the program should not arrive here
+            logging.debug('POKE beyond program size handled by Program class.')
         else:
-            offset -= self.code_start
-            loc = self.bytecode.tell()
+            self.bytecode.seek(offset)
+            self.bytecode.write(int2byte(val))
             # move pointer to end
             self.bytecode.seek(0, 2)
-            if offset > self.bytecode.tell():
-                self.bytecode.write(b'\0' * (offset-self.bytecode.tell()))
-            else:
-                self.bytecode.seek(offset)
-            self.bytecode.write(int2byte(val))
-            self.bytecode.seek(0, 2)
             self.rebuild_line_dict()
-            # restore program pointer
-            self.bytecode.seek(loc)
+        # restore program pointer
+        self.bytecode.seek(loc)
