@@ -2,7 +2,7 @@
 PC-BASIC - strings.py
 String values
 
-(c) 2013--2022 Rob Hagemans
+(c) 2013--2023 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -164,7 +164,10 @@ class StringSpace(object):
 
     def __repr__(self):
         """Debugging representation of string table."""
-        return '\n'.join('%x: %r' % (n, v) for n, v in iteritems(self._strings))
+        return '\n'.join(
+            '%x: %r' % (n, bytes(v))
+            for n, v in iteritems(self._strings)
+        )
 
     def clear(self):
         """Empty string space."""
