@@ -2,7 +2,7 @@
 PC-BASIC - compat.win32
 Interface for Windows system calls
 
-(c) 2018--2022 Rob Hagemans
+(c) 2018--2023 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -57,8 +57,12 @@ def _get_oem_encoding():
 
 
 # if starting from a console, shell will inherit its codepage
-# if starting from the gui (stdin.encoding == None), we're using OEM codepage
-SHELL_ENCODING = sys.stdin.encoding or _get_oem_encoding()
+# if starting from the gui (stdin is None), we're using OEM codepage
+OEM_ENCODING = _get_oem_encoding()
+SHELL_ENCODING = None
+if sys.stdin:
+    SHELL_ENCODING = sys.stdin.encoding
+SHELL_ENCODING = SHELL_ENCODING or OEM_ENCODING
 # there's also an ACP codepage - this seems to be locale.getpreferredencoding()
 #_ACP_ENCODING = 'cp' + str(cdll.kernel32.GetACP())
 
