@@ -2,7 +2,7 @@
 PC-BASIC - config.py
 Configuration file and command-line options parser
 
-(c) 2013--2022 Rob Hagemans
+(c) 2013--2023 Rob Hagemans
 This file is released under the GNU GPL version 3 or later.
 """
 
@@ -14,7 +14,6 @@ import zipfile
 import locale
 import shutil
 import codecs
-import pkg_resources
 from collections import deque
 
 from .compat import iteritems, text_type, iterchar
@@ -25,8 +24,8 @@ from .compat import split_quoted, split_pair
 from .compat import console, IS_CONSOLE_APP, stdio
 from .compat import TemporaryDirectory
 
-from .data import CODEPAGES, FONTS, PROGRAMS
-from .basic import VERSION, NAME, ICON
+from .data import CODEPAGES, FONTS, PROGRAMS, ICON
+from .basic import VERSION, NAME
 from . import data
 
 
@@ -109,6 +108,9 @@ PRESETS = {
         u'prevent-close': u'True',
         u'ctrl-c-break': u'False',
     },
+    u'gwbasic': {
+        u'syntax': u'gwbasic',
+    },
     u'basica': {
         u'reserved-memory': u'789',
     },
@@ -294,7 +296,7 @@ ARGUMENTS = {
     u'mount': {u'type': u'string', u'list': u'*', u'default': [],},
     u'resume': {u'type': u'bool', u'default': False,},
     u'syntax': {
-        u'type': u'string', u'choices': (u'advanced', u'pcjr', u'tandy'),
+        u'type': u'string', u'choices': (u'advanced', u'gwbasic', u'pcjr', u'tandy'),
         u'default': u'advanced',
     },
     u'term': {u'type': u'string', u'default': u'',},
@@ -349,6 +351,8 @@ class Lumberjack(object):
 
     def __init__(self):
         """Set up the global logger temporarily until we know the log stream."""
+        # include messages from warnings madule in the logs
+        logging.captureWarnings(True)
         # we use the awkward logging interface as we can only use basicConfig once
         # get the root logger
         root_logger = logging.getLogger()
@@ -514,8 +518,10 @@ class Settings(object):
         if self._session_params:
             return self._session_params
         # don't parse any options on --resume
+        # except redirects
         if self.get('resume'):
-            return {}
+            params = self._add_implicit_redirects()
+            return params
         # preset PEEK values
         peek_values = {}
         try:
@@ -597,9 +603,8 @@ class Settings(object):
         return params
 
     def _get_redirects(self):
-        """Determine which i/o streams to attach."""
+        """Determine which i/o streams to attach based on config choices."""
         input_streams, output_streams = [], []
-        # explicit redirects
         # input redirects
         infile_params = self.get('input').split(u':')
         if infile_params[0].upper() in (u'STDIO', u'STDIN'):
@@ -634,7 +639,12 @@ class Settings(object):
                     output_streams.append(io.open(outfile, 'ab' if append else 'wb'))
                 except EnvironmentError as e:
                     logging.warning(u'Could not open output file `%s`: %s', outfile, e.strerror)
-        # implicit stdio redirects
+        return self._add_implicit_redirects(input_streams, output_streams)
+
+    def _add_implicit_redirects(self, input_streams=(), output_streams=()):
+        """Determine which i/o streams to attach implicitly."""
+        input_streams = list(input_streams)
+        output_streams = list(output_streams)
         # add stdio if redirected or no interface
         if stdio.stdin not in input_streams and stdio.stdin.buffer not in input_streams:
             if IS_CONSOLE_APP and not stdio.stdin.isatty():
@@ -939,6 +949,7 @@ class ArgumentParser(object):
         # apply shorthands
         for short_arg, replacement in iteritems(SHORTHAND):
             key = key.replace(short_arg, replacement)
+        long_arg_value = None
         for i, short_arg in enumerate(key[1:]):
             try:
                 long_arg, long_arg_value = SHORT_ARGS[short_arg]
@@ -1099,7 +1110,7 @@ class ArgumentParser(object):
             # use utf_8_sig to ignore a BOM if it's at the start of the file
             # (e.g. created by Notepad)
             with io.open(config_file, 'r', encoding='utf_8_sig', errors='replace') as f:
-                if PY2:
+                if PY2: # pragma: no cover
                     config.readfp(WhitespaceStripper(f))
                 else:
                     config.read_file(WhitespaceStripper(f))
